<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version and init hooks to
 * enqueue the admin-specific functionalities.
 *
 * @since      1.0.0
 * @package    JoinChatCTAExtras
 * @author     Creame <hola@crea.me>
 */
class JoinChatCTAExtrasAdmin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set init hook.
	 *
	 * @since    1.0.0
	 * @param    string $plugin_name       The name of this plugin.
	 * @param    string $version           The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		add_action( 'joinchat_run_pre', array( $this, 'init' ) );

	}

	/**
	 * Initialize all hooks
	 *
	 * @since    1.0.0
	 * @param    array $joinchat       JoinChat object.
	 * @return   void
	 */
	public function init( $joinchat ) {

		$loader = $joinchat->get_loader();

		$loader->add_filter( 'joinchat_tab_general_sections', $this, 'get_sections' );
		$loader->add_filter( 'joinchat_field_output', $this, 'fields_output', 10, 3 );

		$loader->add_action( 'load-settings_page_joinchat', $this, 'help_tab' );
		$loader->add_action( 'load-toplevel_page_joinchat', $this, 'help_tab' );

		// Upgrade notice.
		$loader->add_action( 'admin_notices', $this, 'premium_upgrade_notice' );
		$loader->add_action( 'wp_ajax_joinchat_ctaextras_upgrade_notice_dismiss', $this, 'ajax_notice_dismiss' );

	}

	/**
	 * Add White Label setting and help for extended syntax
	 *
	 * @since    1.0.0
	 * @param    array $sections       sections and fields.
	 * @return   array
	 */
	public function get_sections( $sections ) {

		if ( isset( $sections['chat']['message_text'] ) ) {
			$sections['chat']['message_text'] .= '<div class="joinchat_vars_help">' . __( 'Extended Syntax', 'joinchat-cta-extras' ) .
				' <a class="joinchat-show-help" href="#tab-link-cta-extras" title="' . __( 'Show Help', 'creame-whatsapp-me' ) . '">?</a></div>';
		}

		return $sections;

	}

	/**
	 * Joinchat CTA Extras fields HTML output
	 *
	 * @since    1.0.0
	 * @param    string $output       current field output.
	 * @param    string $field_id     current field id.
	 * @param    array  $settings     current joinchat settings.
	 * @return   string
	 */
	public function fields_output( $output, $field_id, $settings ) {

		// Remove addon link.
		if ( 'message_text' === $field_id && false !== strpos( $output, '<p class="joinchat-addon">' ) ) {
			$output = substr( $output, 0, strpos( $output, '<p class="joinchat-addon">' ) );
		}

		// Make textareas wider.
		if ( strpos( $field_id, 'message_text' ) === 0 ) {
			$output = str_replace( 'regular-text', 'large-text', $output );
		}

		return $output;

	}

	/**
	 * Add a help tab to the options page in the WordPress admin
	 *
	 * @since    3.0.0
	 * @access   public
	 * @return   void
	 */
	public function help_tab() {
		$screen = get_current_screen();

		$screen->add_help_tab(
			array(
				'id'      => 'cta-extras',
				'title'   => __( 'Extended Syntax', 'joinchat-cta-extras' ),
				'content' =>
					'<p>' . __( 'In addition to WhatsApp formatting styles _<em>italic</em>_ *<strong>bold</strong>* ~<del>strikethrough</del>~ with <strong>Joinchat CTA Extras</strong> you can enrich your Calls to Action with links, images, videos and more.', 'joinchat-cta-extras' ) . '</p>' .
					'<p>' . __( 'List of supported elements, if you are familiar with Markdown you can use its syntax (<i>italic optional fields</i>):', 'joinchat-cta-extras' ) . '</p>' .
					'<p style="line-height:1.8em;">' .
					'<span>' . __( 'Link', 'joinchat-cta-extras' ) . ' ➜ <code>[<i>title</i>](https://www.example.com)</code> ' . __( 'or', 'joinchat-cta-extras' ) . ' <code>{LINK https://www.example.com <i>title</i>}</code></span><br>' .
					'<span>' . __( 'Button', 'joinchat-cta-extras' ) . ' ➜ <code>{BTN https://www.example.com <i>title</i>}</code></span><br>' .
					'<span>' . __( 'Image', 'joinchat-cta-extras' ) . ' ➜ <code>![<i>alt_text</i>](image.jpg <i>width</i>)</code> ' . __( 'or', 'joinchat-cta-extras' ) . ' <code>{IMG image.jpg <i>width alt_text</i>}</code></span><br>' .
					'<span>' . __( 'Iframe', 'joinchat-cta-extras' ) . ' ➜ <code>#[<i>height</i>](https://www.example.com)</code> ' . __( 'or', 'joinchat-cta-extras' ) . ' <code>{IFRAME https://www.example.com <i>height</i>}</code></span><br>' .
					'<span>' . __( 'Youtube Video', 'joinchat-cta-extras' ) . ' ➜ <code>https://www.youtube.com/watch?v=videoid</code> ' . __( 'or', 'joinchat-cta-extras' ) . ' <code>https://youtu.be/videoid</code></span><br>' .
					'<span>' . __( 'Vimeo Video', 'joinchat-cta-extras' ) . ' ➜ <code>https://vimeo.com/videoid</code><br>' .
					'<span>' . __( 'Calendly calendar widget', 'joinchat-cta-extras' ) . ' ➜ <code>https://calendly.com/username/event</code><br>' .
					'<span>' . __( 'Code', 'joinchat-cta-extras' ) . ' ➜ <code>`code`</code></span><br>' .
					'<span>' . __( 'Horizontal Rule', 'joinchat-cta-extras' ) . ' ➜ <code>---</code></span><br>' .
					'<span>' . __( 'New Message Bubble', 'joinchat-cta-extras' ) . ' ➜ <code>===</code></span>' .
					'</p>' .
					'<p>' . __( 'The elements can be combined. For example, for an image with link:', 'joinchat-cta-extras' ) . ' <code>{LINK https://www.example.com {IMG image.jpg alt_text}}</code></p>' .
					'<p>' . __( 'On images you can use the image url or the image ID of your Media Library. <strong>We recommend</strong> to use the ID that resizes the image to the exact size of the chat window.', 'joinchat-cta-extras' ) . '</p>' .
					'<p>' . __( 'Can use <strong>FEATURED</strong> or <strong>THUMB</strong> as image url to use current post featured image, e.g.', 'joinchat-cta-extras' ) . ' <code>{IMG THUMB}</code> ' . __( 'or', 'joinchat-cta-extras' ) . ' <code>![{PRODUCT}](FEATURED)</code></p>' .
					'<p>' . sprintf( __( 'To learn more, visit our <a href="%s" rel="external" target="_blank">examples and tricks page</a>.', 'joinchat-cta-extras' ), esc_url( 'https://join.chat/cta-extras-examples/' ) ) . '</p>',
			)
		);

	}

	/**
	 * Show admin license notice
	 *
	 * @access   public
	 * @return   void
	 */
	public function premium_upgrade_notice() {

		if ( ( defined( 'DISABLE_NAG_NOTICES' ) && DISABLE_NAG_NOTICES )               // Disabled notices.
			|| ! current_user_can( JoinchatUtil::capability() )                          // User can't activate.
			|| time() < (int) get_option( 'joinchat_ctaextras_upgrade_notice_dismiss' )  // Dismissed.
		) {
			return;
		}

		$link = sprintf( '<a href="%s">Joinchat Account</a>', esc_url( 'https://join.chat/en/account/?utm_source=upgrade&utm_medium=wpadmin&utm_campaign=ctaextras' ) );
		$message = sprintf(
			'%s is now included in %s and will no longer receive updates. Download the new plugin with all Joinchat add-ons from your %s.',
			'<strong>Joinchat CTA Extras</strong>',
			'<strong>Joinchat Premium</strong>',
			$link
		);

		printf( '<div class="notice notice-warning is-dismissible" id="joinchat-ctaextras-notice"><p>%s</p></div>', $message );
		printf(
			'<script>jQuery("#joinchat-ctaextras-notice").on("click", ".notice-dismiss", function () {' .
			'jQuery.post(ajaxurl, { action: "joinchat_ctaextras_upgrade_notice_dismiss", nonce: "%s" }, null, "json");' .
			'});</script>',
			esc_js( wp_create_nonce( 'joinchat_ctaextras_upgrade_nonce' ) )
		);

	}

	/**
	 * Upgtade notice dismiss
	 *
	 * @access   public
	 * @return   void
	 */
	public function ajax_notice_dismiss() {

		check_ajax_referer( 'joinchat_ctaextras_upgrade_nonce', 'nonce', true );
		update_option( 'joinchat_ctaextras_upgrade_notice_dismiss', time() + MONTH_IN_SECONDS, true );
		wp_send_json_success();

	}
}
