<?php

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version and init hooks to
 * enqueue the public-specific functionalities.
 *
 * @since      1.0.0
 * @package    JoinChatCTAExtras
 * @author     Creame <hola@crea.me>
 */
class JoinChatCTAExtrasPublic {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set init hook.
	 *
	 * @since    1.0.0
	 * @param    string $plugin_name       The name of this plugin.
	 * @param    string $version           The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		add_action( 'joinchat_run_pre', array( $this, 'init' ) );

	}

	/**
	 * Initialize all hooks
	 *
	 * @since    1.0.0
	 * @param    array $joinchat       JoinChat object.
	 * @return   void
	 */
	public function init( $joinchat ) {

		$loader = $joinchat->get_loader();

		$loader->add_filter( 'joinchat_format_replacements', $this, 'cta_extras' );
		$loader->add_filter( 'joinchat_content', $this, 'split_messages' );

		$loader->add_action( 'wp_enqueue_scripts', $this, 'register_scripts' );
		$loader->add_action( 'wp_enqueue_scripts', $this, 'enqueue_styles' );

	}

	/**
	 * Add CTA Extras replacements for extended syntax
	 *
	 * @since    1.0.0
	 * @since    1.1.0   Allowed 'width' param on images
	 * @param    array $replacements       current replacements.
	 * @return   array
	 */
	public function cta_extras( $replacements ) {

		$extras = array();

		/**
		 * Image
		 * {IMG image.jpg width alt_text} => <img src="image.jpg" alt="alt_text" width="width">
		 */
		$extras['/{IMG (?<src>[^ }]+)(?: (?<w>\d+))?(?: (?<alt>[^}]+))?}/u'] = array( $this, 'image' );

		/**
		 * Iframe
		 * {IFRAME https://www.example.com height} => <iframe src="https://www.example.com" height="height"></iframe>
		 */
		$extras['/{IFRAME (?<src>[^} ]+)(?: (?<h>\d+))?}/u'] = array( $this, 'iframe' );

		/**
		 * Link
		 * {LINK https://www.example.com title} => <a href="https://www.example.com">title</a>
		 */
		$extras['/{LINK (?<href>[^ }]+)(?: (?<text>[^}]+))?}/u'] = array( $this, 'link' );

		/**
		 * Link button
		 * {BTN https://www.example.com title} => <a class="joinchat__btn" href="https://www.example.com">title</a>
		 */
		$extras['/{BTN (?<href>[^ }]+)(?: (?<text>[^}]+))?}/u'] = array( $this, 'button' );

		/**
		 * Markdown (like) iframe
		 * #[height](https://www.example.com) => <iframe src="https://www.example.com" height="height"></iframe>
		 */
		$extras['/#\[(?<h>\d+)?\]\((?<src>.*?)\)/u'] = array( $this, 'iframe' );

		/**
		 * Markdown image
		 * ![alt text](image.jpg width) => <img src="image.jpg" alt="alt text" width="width">
		 */
		$extras['/!\[(?<alt>.*?)\]\((?<src>[^\) ]*)(?: (?<w>\d+))?\)/u'] = array( $this, 'image' );

		/**
		 * Markdown link
		 * [title](https://www.example.com) => <a href="https://www.example.com">title</a>
		 */
		$extras['/\[(?<text>[^\[\]]+)?\]\((?<href>[^)]+)\)/u'] = array( $this, 'link' );

		/**
		 * Markdown code
		 * `your code` => <code>your code</code>
		 */
		$extras['/(^|\W)`(.+?)`(\W|$)/u'] = '<code>$1</code>';

		/**
		 * Markdown horizontal rule
		 * --- => <hr>
		 */
		$extras['/^-{3,}$/u'] = '<hr>';

		/**
		 * Youtube video (capture youtube urls)
		 */
		$extras['/(?:^|[^"])(?:https?:\/\/(?:www\.youtube\.com\/watch\?v=|youtu.be\/)(?<id>[\w-]+))/u'] = array( $this, 'youtube' );

		/**
		 * Vimeo video (capture vimeo urls)
		 */
		$extras['/(?:^|[^"])(?:https?:\/\/(?:www\.)?vimeo\.com\/(?<id>\d+))/u'] = array( $this, 'vimeo' );

		/**
		 * Calendly calendar widget (capture calendly urls)
		 */
		$extras['/(?:^|[^"])(https?:\/\/calendly\.com\/.+)/u'] = array( $this, 'calendly' );

		return array_merge( $replacements, $extras );

	}

	/**
	 * Link html output with regex groups
	 *
	 * $groups['href']
	 * $groups['text']
	 * $groups['class']
	 *
	 * @since    1.1.0
	 * @since    2.2.0              accept $class param
	 * @since    2.3.0              add rel external
	 * @param    array $groups      regex groups of link.
	 * @return   string html <a>
	 */
	public function link( $groups ) {

		$href  = esc_url( $groups['href'] );
		$text  = isset( $groups['text'] ) && $groups['text'] ? $groups['text'] : preg_replace( '/^https?:\/\//u', '', $href );
		$class = isset( $groups['class'] ) ? 'class="' . esc_attr( $groups['class'] ) . '"' : '';

		// Check if is an external link and add rel attribute for better SEO & Security.
		$rel  = '';
		$host = wp_parse_url( $href, PHP_URL_HOST );
		if ( ! is_null( $host ) && wp_parse_url( get_home_url(), PHP_URL_HOST ) !== $host ) {
			$rel = 'rel="external nofollow noopener noreferrer" target="_blank"';
		}

		$output = "<a $class href=\"$href\" $rel>$text</a>";

		return apply_filters( 'joinchat_cta_out', $output, 'image', $groups );

	}

	/**
	 * Button link html output with regex groups
	 *
	 * $groups['href']
	 * $groups['text']
	 *
	 * @since    2.0.0
	 * @param    array $groups       regex groups of link.
	 * @return   string html <a>
	 */
	public function button( $groups ) {

		$groups['class'] = 'joinchat__btn';

		return $this->link( $groups );

	}

	/**
	 * Image html output with regex groups
	 *
	 * If image url is numeric try to get the image of media library an
	 * resize it to correct Joinchat chat window size.
	 *
	 * $groups['src']
	 * $groups['alt']
	 * $groups['w']
	 *
	 * @since    1.0.0
	 * @since    2.4.0 Added 'FEATURED' or 'THUMB' src for post thumbnail image
	 *
	 * @param    array $groups       regex groups of image.
	 * @return   string html <img>
	 */
	public function image( $groups ) {

		$chat_max = 340; // Chat window max width.

		$src = $groups['src'];
		$alt = 'alt="' . ( isset( $groups['alt'] ) ? esc_attr( $groups['alt'] ) : '' ) . '"';
		$w   = isset( $groups['w'] ) && is_numeric( $groups['w'] ) ? $groups['w'] : $chat_max;

		// If is post featured image.
		if ( in_array( strtoupper( $src ), array( 'THUMB', 'FEATURED' ), true ) ) {
			$src = (int) get_post_thumbnail_id();
		}

		$sizes = array();
		$width = $w < $chat_max ? "width=\"$w\"" : '';
		$class = $w < $chat_max ? 'class="joinchat-inline"' : '';

		if ( is_numeric( $src ) && intval( $src ) > 0 ) {
			$info = wp_get_attachment_image_src( $src, 'full' );

			if ( is_array( $info ) ) {
				if ( 0 === $info[1] ) {
					$sizes[1] = $info[0];
				} else {
					$class = min( $w, $info[1] ) < $chat_max ? 'class="joinchat-inline"' : '';

					if ( apply_filters( 'joinchat_image_original', JoinChatUtil::is_animated_gif( $src ), $src, 'cta' ) ) {
						$sizes[1] = $info[0];
					} else {
						$w_max = min( $w, $chat_max ); // Image max width.
						$ratio = $info[2] / $info[1];  // Image aspect ratio.

						for ( $i = 1; $i < 4; $i++ ) {
							if ( $w_max * $i < $info[1] ) {
								$thumb       = JoinChatUtil::thumb( $src, $w_max * $i, round( $w_max * $i * $ratio ) );
								$sizes[ $i ] = esc_url( is_array( $thumb ) ? $thumb['url'] : $info[0] );
							} else {
								$sizes[ $i ] = esc_url( $info[0] );
								$i           = 4; // end for.
							}
						}
					}
				}
			}
		} elseif ( ! empty( $src ) ) {
			$sizes[1] = esc_url( $src );
		}

		if ( count( $sizes ) ) {
			$srcset = isset( $sizes[2] ) ? "srcset=\"{$sizes[2]} 2x" . ( isset( $sizes[3] ) ? ", {$sizes[3]} 3x\"" : '"' ) : '';

			$output = preg_replace( '/\s+/', ' ', "<img src=\"{$sizes[1]}\" $srcset $alt $width $class loading=\"lazy\">" );
		} else {
			$output = '<code class="not-found">Image not found</code>';
		}

		return apply_filters( 'joinchat_cta_out', $output, 'image', $groups );

	}

	/**
	 * Iframe html output with regex groups
	 *
	 * $groups['src']
	 * $groups['h']
	 *
	 * @since    2.2.0
	 * @param    array $groups       regex groups of iframe.
	 * @return   string html
	 */
	public function iframe( $groups ) {

		$this->enqueue_scripts();

		$src    = esc_url( $groups['src'] );
		$height = isset( $groups['h'] ) && is_numeric( $groups['h'] ) ? $groups['h'] : '';

		$output = "<span class=\"joinchat__iframe\" data-src=\"$src\" data-height=\"$height\"></span>";

		return apply_filters( 'joinchat_cta_out', $output, 'iframe', $groups );

	}

	/**
	 * Youtube video html output with regex groups
	 *
	 * $groups['id']
	 *
	 * @since    2.2.0
	 * @param    array $groups       regex groups of video.
	 * @return   string html
	 */
	public function youtube( $groups ) {

		$this->enqueue_scripts();

		$id    = esc_attr( $groups['id'] );
		$thumb = esc_url( apply_filters( 'joinchat_cta_video_thumbnail', "https://i.ytimg.com/vi/$id/hqdefault.jpg", $id, 'youtube' ) );

		$output = "<span class=\"joinchat__video\" data-service=\"youtube\" data-id=\"$id\" style=\"background-image:url($thumb)\"></span>";

		return apply_filters( 'joinchat_cta_out', $output, 'youtube', $groups );

	}

	/**
	 * Vimeo video html output with regex groups
	 *
	 * $groups['id']
	 *
	 * @since    2.2.0
	 * @param    array $groups       regex groups of video.
	 * @return   string html
	 */
	public function vimeo( $groups ) {

		$this->enqueue_scripts();

		$id  = $groups['id'];
		$ext = isset( $_SERVER['HTTP_ACCEPT'] ) && strpos( $_SERVER['HTTP_ACCEPT'], 'image/webp' ) !== false ? 'webp' : 'jpg';

		// Required Vimeo oEmbed call, added filter to allow disable.
		if ( apply_filters( 'joinchat_cta_vimeo_thumb', true ) ) {

			$thumb = get_transient( "joinchat__vimeo_$id" );

			if ( ! $thumb ) {
				$params  = array(
					'url'   => rawurlencode( "https://vimeo.com/$id" ),
					'width' => 640,
				);
				$oembed  = add_query_arg( $params, 'https://vimeo.com/api/oembed.json' );
				$request = wp_safe_remote_get( $oembed, array( 'timeout' => 5 ) );

				if ( ! is_wp_error( $request ) ) {
					$data  = json_decode( wp_remote_retrieve_body( $request ), true );
					$thumb = $data['thumbnail_url'];
				} else {
					$thumb = "https://i.vimeocdn.com/video/{$id}_640.jpg";
				}

				set_transient( "joinchat__vimeo_$id", $thumb, DAY_IN_SECONDS );
			}
		} else {
			// Can be incorrect thumbnail.
			$thumb = "https://i.vimeocdn.com/video/{$id}_640.jpg";
		}

		$thumb = preg_replace( '/(\.jpg)$/i', ".$ext", $thumb );
		$thumb = esc_url( apply_filters( 'joinchat_cta_video_thumbnail', $thumb, $id, 'vimeo' ) );

		$output = "<span class=\"joinchat__video\" data-service=\"vimeo\" data-id=\"$id\" style=\"background-image:url($thumb)\"></span>";

		return apply_filters( 'joinchat_cta_out', $output, 'vimeo', $groups );

	}

	/**
	 * Calendly html output
	 *
	 * @since    2.6.0
	 * @param    array $groups       regex groups of calendly tag.
	 * @return   string html
	 */
	public function calendly( $groups ) {

		// TODO: optimize make defer or async
		wp_enqueue_script( 'calendly' );

		$output = '<div class="calendly-inline-widget" data-url="' . $groups[0] . '?hide_event_type_details=1&hide_gdpr_banner=1" data-auto-load="false"></div>';

		return apply_filters( 'joinchat_cta_out', $output, 'calendly', $groups );

	}

	/**
	 * Split messages
	 *
	 * @since    2.5.0
	 * @param  string $content CTA content.
	 * @return string
	 */
	public function split_messages( $content ) {

		$content = preg_replace( '/<br>={3,}<br>/u', "</div>\n<div class=\"joinchat__message\">", $content, -1, $count );

		$messages = explode( "\n", $content );

		foreach ( $messages as &$message ) {
			if ( wp_strip_all_tags( $message ) === '' ) {
				$message = str_replace( 'joinchat__message', 'joinchat__message joinchat__message--media', $message );
			}
		}

		$content = implode( "\n", $messages );

		if ( $count > 0 ) {
			$this->enqueue_scripts();

			$content = "<div class=\"joinchat__dialog\">\n" . $content . "\n</div>";
		}

		return $content;

	}

	/**
	 * Register the scripts for the public-facing side of the site.
	 *
	 * Script controls video and iframe loading.
	 *
	 * @since    2.2.0
	 * @return   void
	 */
	public function register_scripts() {

		if ( wp_script_is( 'joinchat', 'enqueued' ) ) {
			$min = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

			wp_register_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . "js/{$this->plugin_name}{$min}.js", array( 'joinchat' ), $this->version, true );
			wp_register_script( 'calendly', 'https://assets.calendly.com/assets/external/widget.js', array(), null, true );
		}

	}

	/**
	 * Enqueue the scripts for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 * @since    2.2.0  require previous register_scripts()
	 * @return   void
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( $this->plugin_name );

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * Add styles for new allowed elements in CTAs.
	 *
	 * @since    1.0.0
	 * @return   void
	 */
	public function enqueue_styles() {

		if ( wp_style_is( 'joinchat', 'enqueued' ) ) {
			$min = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . "css/{$this->plugin_name}{$min}.css", array( 'joinchat' ), $this->version, 'all' );
		}

	}
}
